/*

 PMMIncoming - PMMail/2 Incoming REXX Filter - Version 2.02 - 13.09.2005

 HISTORY

    * 1.0 - (11.03.2003) - first version
    * 1.1 - (11.03.2003) - can change ISO-8859-15 to ISO-8859-1 in header now
    * 1.2 - (15.03.2003) - can add timezone difference for PDT and EDT now
    * 1.3 - (25.09.2003) - recognizes US-ASCII charset now
    * 1.4 - (14.10.2003) - can check ISO encoding for Reply-To now
    * 1.5 - (   10.2003) - can check semicolon for Multipart/Alternative now
    * 1.6 - (06.11.2003) - ISO-8859-15 to ISO-8859-1 change for Subject
                           fixed for capital letters
    * 1.7 - (06.03.2004) - can add charset to Content-Type now if Content-Type=
                           text/plain without a charset is given
    * 1.8 - (06.02.2005) - can check encoding of some Multipart Mails now (just
                           the text-part)
    * 1.9 - (01.09.2005) - (internal version) can convert UTF-8 header lines
                           now
    * 2.0 - (09.09.2005) - all checks for the header have been completely
                           rewritten:
                           - now Subject, From, To, Cc and Reply-To are being
                             checked for correct and usable encoding
                           - encoding conversion for both ISO-8859-15 and
                             UTF-8 is possible now
                           - it can now convert regardless of position of
                             "encoding" in the headerline
                           - now multi-lined header entries are fully handled
                             (e.g. more than one recipient or a two-line
                             subject)
                           - the semicolon check for Multipart/Alternative was
                             removed - have never seen this problem in the wild
                           - a couple of bugs with very exotic headers are
                             fixed now
     * 2.01 - (11.09.2005) - fixed a bug in timezone handling that was intro-
                             duced in 2.0
     * 2.02 - (13.09.2005) - fixed a bug in timezone handling with CEST
                             accidently being falsely fixed
                           - added conversion of mails that are marked as
                             text/html without any charset and transfer-enc
                           - added fix for iso-encoded mails being falsely
                             marked 7bit (e.g. by Aldi)

 INTENTION OF THIS SCRIPT

    This script helps solving problems with mails that do not comply with
    standards or which use "newer" encodings and which can therefore not be
    handled/displayed correctly by PMMail/2.
    It began with just a few non-standard mails a couple of years ago, but now
    almost every second mail I get is already "mutated" and does not give a
    damn about standards or uses encodings that PMMail/2 is not able to handle
    - like UTF-8.
    Most problems I have when it comes to encodings occour with German Umlauts
    or Ligatures. Therefore this script addresses German users of PMMail/2 in
    the first place. But since it corrects quite a few other problems as well
    it can be used by every PMMail/2 user that is left on earth...

 CAPABILITIES OF THIS SCRIPT

    The following main problems are being addressed by this script:

    * Subject, From, To, Cc and Reply-To are transformed to ISO-8859-1 if
      one of the following problems occour:
      - headerline is encoded in UTF-8 (idicated)
      - headerline is encoded in ISO-8859-15 (idicated)
      - headerline has no indicated encoding but is UTF-8
      - headerline has no indicated encoding but is ISO-8859-15
      (this all would otherwise lead to false Subjects, Froms, etc. if German
       Umlauts, etc are used)
    * Body encoding is changed to ISO-8859-1 if Content-Type indicates
      ISO-8859-15
      (this would otherwise lead to a false Body if German Umlauts, etc. are
       used)
    * Mime-Version, Content-Type and Content-Transfer-Encoding are added to
      the header if not present
      (this would otherwise lead to a false Body if German Umlauts, etc. are
       used, cause in most cases these mails use ISO-8859-1 encoding - which
       has to be indicated for PMMail/2 to be recognized)
    * ISO encoded mails that are falsely set to a 7bit tranfer encoding are
      set to 8bit
      (this would otherwise lead to a false body)
    * Date is adjusted by adding the time difference between used timezone and
      GMT if PST (PDT) or EST (EDT) is used and no time difference is idicated.
      (this would otherwise lead to a misinterpretation of the message's
       timestamp)
    * Sub-Header in some Multipart mails are checked as well if they contain
      text and have incomplete headerinformation
      (this would otherwise again lead to false Body if German Umlauts, etc.
       are used)

    Missing header entries are added and incomplete entries are adjusted.
    While doing this one cannot rule out that in some mails this means the
    added or adjusted entries are not necessary. But according to my experience
    they don't do any harm either!

    If this script does any manipulation to the incoming message, it indicates
    it with a beep and adds a headerline that describes what has been done.

 INSTALLATION

    To let this script check every incoming message before it is dropped into
    your inbox, enter it in "Message Receive Exit" in your Account Settings
    on the REXX page.

    Message Receive Exit -> X:\PMMail\REXX\PMMIncoming.cmd

 DISCLAIMER

    YOU ARE USING THIS PROGRAM AT YOUR OWN RISK! It is provided 'AS IS' and
    without any warranties! I don't take any responsibillity for damages,
    problems, custodies, marital disputes, etc. resulting from use, inability
    to use, misuse, possession or non-possession of this program directly or
    indirectly. I also don't give any warranty for bug-free operation, fitness
    for a particular purpose or the appropriate behaviour of the program
    concerning animals, programers and little children.

 THANKS

    For the implementation of CountStr procedure I did make good use of Dallas
    Legan's work (Southern California OS/2 User Group [SCOUG]).
     -> http://www.scoug.com/os24u/2002/scoug202.changestr.html

    The basic structure of this script is inspired by the Spamcheck Skript of
    Winfried Tilanus.
     -> http://www.xs4all.nl/~wtilanus/spam.html

 CONTACT

    This script is freeware. But it would be very nice if you would at least
    let me know that you're using it.

    eMail: Axel Hagedorn <axel.hagedorn@gmx.de>

    Suggestions for improvements and bug-reports are always welcome (well the
    bug-reports maybe not always ;-).

 COPYRIGHT

    2005 - INFINITY/2 - Axel Hagedorn, Darmstadt, Germany

*/

CALL RxFuncadd 'SysFileDelete', 'RexxUtil', 'SysFileDelete'

PARSE ARG mailfile

/* Read mailfile */
DO line = 1 BY 1 WHILE LINES(mailfile)
  mail.line = LINEIN(mailfile)
  shadow.line = ''
END

maillength = line - 1 /* Number of lines in Mailfile */
newentries = 0        /* Number of new lines in mail header */

mimever = 0           /* MIME-Type not present */
ctype = 0             /* Content-Type not present */
ctypechar = 0         /* Content-Type contains no charset setting */
ctransenc = 0         /* Content-Transfer-Encoding not present */
textplain = 0
texthtml = 0
sevenbit = 0
isoenc = 0

boundary = ''

DO line = 1 BY 1 WHILE LENGTH(mail.line) \== 0 /* Walk through header */

  /* Get headerentry-name */
  headername = ''
  headerline = ''
  PARSE VAR mail.line headername ': ' headerline
  PARSE UPPER VAR headername headernameupper
  PARSE UPPER VAR headerline headerlineupper

  /* Get number of lines the headerentry uses in the header */
  /* (Check for space and tab) */
  DO inline = line + 1 BY 1 WHILE ((LEFT(mail.inline,1) = X2C('09') | LEFT(mail.inline,1) = " ") & LENGTH(mail.inline) > 0);
  END
  inline = inline - line /* inline tells you the number of lines (mostly 1) */

  /* Do conversions on headerentries */
  SELECT

    /* Subject/From/To/Cc/Reply-To handling */
    WHEN (headernameupper = 'SUBJECT' | headernameupper = 'FROM' | headernameupper = 'TO' | headernameupper = 'CC' | headernameupper = 'REPLY-TO') THEN
      DO i = 1 TO inline /* Walk through headerentry lines */
        mail.line = ConvHdrLine(headername,headerline,i,inline,mail.line,newentries)
        IF newentry \== '' THEN entry.newentries = newentry
        IF (inline > 1 & i < inline) THEN
          DO
            line = line + 1
            headerline = STRIP(mail.line)
            PARSE UPPER VAR headerline headerlineupper
          END
      END

    /* Date handling - PST/PDT and EST/EDT timezone fixup (used by ebay and aol)*/
    WHEN headernameupper = 'DATE' THEN
      DO
        PARSE VAR headerline part1 ':' part2 ':' part3 ' ' timezone
        SELECT
          WHEN (CountStr('PST',timezone) > 0 & CountStr('0',timezone) = 0) THEN timezone = '-0800 (PST)'
          WHEN (CountStr('PDT',timezone) > 0 & CountStr('0',timezone) = 0) THEN timezone = '-0700 (PDT)'
          WHEN (CountStr('EST',timezone) > 0 & CountStr('CEST',timezone) = 0 & CountStr('0',timezone) = 0) THEN timezone = '-0500 (EST)'
          WHEN (CountStr('EDT',timezone) > 0 & CountStr('0',timezone) = 0) THEN timezone = '-0400 (EDT)'
          OTHERWISE
            DO
              IF inline > 1 THEN line = line + inline - 1 /* just in case headerentry has more than one line */
              ITERATE
            END
        END
        mail.line = 'Date: ' || part1 || ':' || part2 || ':' || part3 || ' ' || timezone
        newentries = newentries + 1
        entry.newentries = 'X-INFINITY/2-Modified: Date fully qualified - "' || timezone || '" added'
        CALL BEEP 900,100
        IF inline > 1 THEN line = line + inline - 1 /* just in case headerentry has more than one line */
      END

    WHEN headernameupper = 'CONTENT-TYPE' THEN
      DO
        ctype = 1
        DO i = 1 TO inline /* Walk through headerentry lines */
          /* is a Charset set? */
          IF CountStr('CHARSET=',headerlineupper) > 0 THEN ctypechar = 1
          /* is the Charset US-ASCII? */
          IF CountStr('US-ASCII',headerlineupper) > 0 THEN ctypechar = 2
          IF CountStr('8859-1',headerlineupper) > 0 THEN isoenc = 1
          /* is Content-Type TEXT/PLAIN? */
          IF CountStr('TEXT/PLAIN;',headerlineupper) > 0 THEN textplain = 1
          IF CountStr('TEXT/HTML;',headerlineupper) > 0 THEN texthtml = 1
          /* change 8859-15 to 8859-1 */
          IF CountStr('8859-15',headerlineupper) > 0 THEN
            DO
              mail.line = ReplStr('8859-15','8859-1',mail.line)
              newentries = newentries + 1
              entry.newentries = 'X-INFINITY/2-Modified: Content-Type line ' || i || ' of ' || inline || ' changed from ISO-8859-15 to ISO-8859-1'
              CALL BEEP 900,100
            END
          /* save boundary for body-scan */
          IF (CountStr('BOUNDARY',headerlineupper) > 0) THEN PARSE VAR mail.line part1 '="' boundary '"' part3
          IF (inline > 1 & i < inline) THEN
            DO
              line = line + 1
              headerline = STRIP(mail.line)
              PARSE UPPER VAR headerline headerlineupper
            END
        END
        /* add ISO-8859-1 Charset if Text/Plain is given without Charset */
        IF (ctypechar = 0 & (textplain = 1 | texthtml = 1)) THEN
          DO
            shadow.line = ' charset="iso-8859-1"'
            newentries = newentries + 1
            entry.newentries = 'X-INFINITY/2-Added: charset="iso-8859-1" added to Content-Type'
            CALL BEEP 900,100
          END
      END

    /* is MIME-Version present? */
    WHEN headernameupper = 'MIME-VERSION' THEN
      DO
        mimever = 1
        IF inline > 1 THEN line = line + inline - 1 /* just in case Headerentry has more than one line */
      END

    /* is Content-Transfer-Encoding present? */
    WHEN headernameupper = 'CONTENT-TRANSFER-ENCODING' THEN
      DO
        ctransenc = 1
        /* is a 7Bit encoding set - can be wrong!? */
        IF CountStr('7BIT',headerlineupper) > 0 THEN sevenbit = line
        IF inline > 1 THEN line = line + inline - 1 /* just in case Headerentry has more than one line */
      END
    OTHERWISE IF inline > 1 THEN line = line + inline - 1 /* just in case Headerentry has more than one line */
  END
END

/* MIME-Version not set, MIME-Version: 1.0 will be set */
IF (mimever = 0) THEN
  DO
    newentries = newentries + 1
    entry.newentries = 'MIME-Version: 1.0'
    newentries = newentries + 1
    entry.newentries = 'X-INFINITY/2-Added: MIME-Version: 1.0'
    CALL BEEP 900,100
  END

/* Content-Type is not set
   Content-Type: text/plain; charset="iso-8859-1" will be set */
IF ctype = 0 THEN
  DO
    newentries = newentries + 1
    entry.newentries = 'Content-Type: text/plain; charset="iso-8859-1"'
    newentries = newentries + 1
    entry.newentries = 'X-INFINITY/2-Added: Content-Type: text/plain; charset="iso-8859-1"'
    CALL BEEP 900,100
    ctypechar = 1
  END

/* Content-Transfer-Encoding is not set,
   Content-Transfer-Encoding: 8bit will be set if no US-ASCII */
IF (ctypechar > 0 & ctransenc = 0) THEN
  DO
    newentries = newentries + 1
    IF (ctypechar = 1) THEN entry.newentries = 'Content-Transfer-Encoding: 8bit'
      ELSE entry.newentries = 'Content-Transfer-Encoding: 7bit'
    newentries = newentries + 1
    IF (ctypechar = 1) THEN entry.newentries = 'X-INFINITY/2-Added: Content-Transfer-Encoding: 8bit'
      ELSE entry.newentries = 'X-INFINITY/2-Added: Content-Transfer-Encoding: 7bit'
    CALL BEEP 900,100
  END

/* Content-Transfer-Encoding is set falsely to 7bit with an ISO encoding,
   Content-Transfer-Encoding: 8bit will be set */
IF (sevenbit > 0 & isoenc = 1) THEN
  DO
    newentries = newentries + 1
    mail.sevenbit = ReplStr('7','8',mail.sevenbit)
    entry.newentries = 'X-INFINITY/2-Modified: Content-Transfer-Encoding: 7bit changed to 8bit'
    CALL BEEP 900,100
  END


headerlength = line /* number of lines in Mail-Header */

/* Boundary was found - now checking text-part for compatible encoding - that means body needs to be scanned */
IF (boundary > '') THEN
  DO
    boundary = '--' || boundary
    DO line = headerlength TO maillength
      IF mail.line = boundary THEN
         DO
          lineone = line + 1
          PARSE UPPER VAR mail.lineone part1 'TEXT/PLAIN' part2
          IF part2 > 0 THEN
            DO
              /*no charset?*/
              PARSE UPPER VAR mail.lineone part1 'CHARSET' part2
              linetwo = lineone + 1
              PARSE UPPER VAR mail.linetwo part1 'CHARSET' part3
              IF (part2 = '') & (part3 = '') THEN
                DO
                  mail.lineone = 'Content-Type: text/plain; charset="iso-8859-1"'
                  newentries = newentries + 1
                  entry.newentries = 'X-INFINITY/2-Modified: charset="iso-8859-1" added to Content-Type of text/plain mailpart'
                  CALL BEEP 900,100
                  CALL BEEP 900,100
                END
                ELSE
                  IF CountStr('8859-15',part2) > 0 THEN
                    DO
                      mail.lineone = ReplStr('8859-15','8859-1',mail.lineone)
                      newentries = newentries + 1
                      entry.newentries = 'X-INFINITY/2-Modified: ISO-8859-15 changed to ISO-8859-1 in text/plain mailpart'
                      CALL BEEP 900,100
                      CALL BEEP 900,100
                    END
                    ELSE
                      IF CountStr('8859-15',part3) > 0 THEN
                        DO
                          mail.linetwo = ReplStr('8859-15','8859-1',mail.linetwo)
                          newentries = newentries + 1
                          entry.newentries = 'X-INFINITY/2-Modified: ISO-8859-15 changed to ISO-8859-1 in text/plain mailpart'
                          CALL BEEP 900,100
                          CALL BEEP 900,100
                        END
              PARSE UPPER VAR mail.lineone . 'CONTENT-TRANSFER-ENCODING: ' encoding
              IF LENGTH(encoding) = 0 THEN
                DO
                  PARSE UPPER VAR mail.linetwo . 'CONTENT-TRANSFER-ENCODING: ' encoding
                  IF LENGTH(encoding) = 0 THEN
                    DO
                      linethree = linetwo + 1
                      PARSE UPPER VAR mail.linethree . 'CONTENT-TRANSFER-ENCODING: ' encoding
                      IF LENGTH(encoding) = 0 THEN
                        DO
                          shadow.line = 'Content-Transfer-Encoding: 8bit'
                          newentries = newentries + 1
                          entry.newentries = 'X-INFINITY/2-Added: Content-Transfer-Encoding: 8bit to text/plain mailpart'
                          CALL BEEP 900,100
                          CALL BEEP 900,100
                        END
                    END
                END
            END
        END
    END
  END

/* Close mailfile */
rc = STREAM(mailfile, 'c', 'close')

/* Save modified mailfile */
IF newentries > 0 THEN
  DO
    rc = SysFileDelete(mailfile)
    DO line = 1 TO maillength
      IF line = headerlength THEN
        DO newline = 1 to newentries
          rc = lineout(mailfile, entry.newline)
        END
      rc = lineout(mailfile, mail.line)
      IF shadow.line > '' THEN rc = lineout(mailfile, shadow.line)
    END
  END

EXIT


/* Procedure ConvHdrLine - Converting headerline */
ConvHdrLine:PROCEDURE EXPOSE newentries newentry

PARSE ARG headername, headerline, i, inline, mailline, newentries

newentry = ''
/* Transforming headerentry from ISO-8859-1 (8bit) or UTF-8 (8bit) to ISO-8859-1 (qp) */
IF CountStr('=?',headerline)+CountStr('?q?',headerline)+CountStr('?Q?',headerline)+CountStr('?=',headerline) < 3 THEN
  DO
    newheaderline = ISO8b2ISOqp(headerline)
    IF headerline \== newheaderline THEN
      DO
        IF i = 1 THEN mailline = headername || ': ' || '=?iso-8859-1?q?' || newheaderline || '?='
        ELSE mailline = ' ' || '=?iso-8859-1?q?' || newheaderline || '?='
        newentries = newentries + 1
        newentry = 'X-INFINITY/2-Modified: ' || headername || ' line ' || i || ' of ' || inline || ' transformed from ISO-8859-1 (8bit) to ISO-8859-1 (qp)'
        CALL BEEP 900,100
      END
    ELSE
      DO
        newheaderline = UTF8b2ISOqp(headerline)
        IF headerline \== newheaderline THEN
          DO
            IF i = 1 THEN mailline = headername || ': ' || '=?iso-8859-1?q?' || newheaderline || '?='
            ELSE mailline = ' ' || '=?iso-8859-1?q?' || newheaderline || '?='
            newentries = newentries + 1
            newentry = 'X-INFINITY/2-Modified: ' || headername || ' line ' || i || ' of ' || inline || ' transformed from UTF-8 (8bit) to ISO-8859-1 (qp)'
            CALL BEEP 900,100
          END
      END
  END
/* Changing headerentry from ISO-8859-15 (qp) to ISO-8859-1 (qp) */
ELSE
  DO
    PARSE UPPER VAR headerline preiso15headerline '=?ISO-8859-15?Q?' iso15headerline '?=' postiso15headerline
    IF iso15headerline > '' THEN
      DO
        IF i = 1 THEN mailline = headername || ': ' || '=?iso-8859-1?q?' || SUBSTR(headerline,1,LENGTH(preiso15headerline)) || SUBSTR(headerline, LENGTH(preiso15headerline) + 17, LENGTH(iso15headerline)) || SUBSTR(headerline, LENGTH(headerline) - LENGTH(postiso15headerline) + 1) || '?='
        ELSE mailline = ' ' || '=?iso-8859-1?q?' || SUBSTR(headerline,1,LENGTH(preiso15headerline)) || SUBSTR(headerline, LENGTH(preiso15headerline) + 17, LENGTH(iso15headerline)) || SUBSTR(headerline, LENGTH(headerline) - LENGTH(postiso15headerline) + 1) || '?='
        newentries = newentries + 1
        newentry = 'X-INFINITY/2-Modified: ' || headername || ' line ' || i || ' of ' || inline || ' changed from ISO-8859-15 (qp) to ISO-8859-1 (qp)'
        CALL BEEP 900,100
      END
    /* Transforming headerentry from UTF-8 (qp) to ISO-8859-1 (qp) */
    ELSE
      DO
        PARSE UPPER VAR headerline preutf8headerline '=?UTF-8?Q?' utf8headerline '?=' postutf8headerline
        IF utf8headerline > '' THEN
          DO
            utf8headerline = SUBSTR(headerline,1,LENGTH(preutf8headerline)) || SUBSTR(headerline, LENGTH(preutf8headerline) + 11, LENGTH(utf8headerline)) || SUBSTR(headerline, LENGTH(headerline) - LENGTH(postutf8headerline) + 1)
            newheaderline = UTFqp2ISOqp(utf8headerline)
            IF utf8headerline \== newheaderline THEN
              DO
                If i = 1 THEN mailline = headername || ': ' || '=?iso-8859-1?q?' || newheaderline || '?='
                ELSE mailline = ' ' || '=?iso-8859-1?q?' || newheaderline || '?='
                newentries = newentries + 1
                newentry = 'X-INFINITY/2-Modified: ' || headername || ' line ' || i || ' of '  || inline || ' transformed from UTF-8 (qp) to ISO-8859-1 (qp)'
                CALL BEEP 900,100
              END
          END
      END
  END

RETURN mailline


/* Procedure UTFqp2ISOqp - Replacing UTF-8 encoded (qp) umlauts with ISO-8859-1 (qp) */
UTFqp2ISOqp: PROCEDURE

PARSE ARG string

IF string = '' THEN RETURN ''

string = ReplStr('=C3=9F','=DF',string) /* sz */
string = ReplStr('=C3=A4','=E4',string) /* ae */
string = ReplStr('=C3=B6','=F6',string) /* oe */
string = ReplStr('=C3=BC','=FC',string) /* ue */
string = ReplStr('=C3=84','=C4',string) /* AE */
string = ReplStr('=C3=96','=D6',string) /* OE */
string = ReplStr('=C3=9C','=DC',string) /* UE */
string = ReplStr('=E2=82=AC','EUR',string) /* Euro-Symbol */

RETURN string


/* Procedure UTF8b2ISOqp - Replacing UTF-8 encoded (8bit) umlauts with ISO-8859-1 (qp) */
UTF8b2ISOqp: PROCEDURE

PARSE ARG string

IF string = '' THEN RETURN ''

string = ReplStr('ß','=DF',string) /* sz */
string = ReplStr('ä','=E4',string) /* ae */
string = ReplStr('ö','=F6',string) /* oe */
string = ReplStr('ü','=FC',string) /* ue */
string = ReplStr('Ä','=C4',string) /* AE */
string = ReplStr('Ö','=D6',string) /* OE */
string = ReplStr('Ü','=DC',string) /* UE */
string = ReplStr('€','EUR',string) /* Euro-Symbol */

RETURN string


/* Procedure UTF8b2ISO8b - Replacing UTF-8 encoded (8bit) umlauts with ISO-8859-1 (8bit) */
UTF8b2ISO8b: PROCEDURE

PARSE ARG string

IF string = '' THEN RETURN ''

string = ReplStr('ß','',string) /* sz */
string = ReplStr('ä','',string) /* ae */
string = ReplStr('ö','',string) /* oe */
string = ReplStr('ü','',string) /* ue */
string = ReplStr('Ä','',string) /* AE */
string = ReplStr('Ö','',string) /* OE */
string = ReplStr('Ü','',string) /* UE */
string = ReplStr('€','EUR',string) /* Euro-Symbol */

RETURN string


/* Procedure ISO8b2ISOqp - Replacing ISO-8859-1 encoded (8bit) umlauts with ISO-8859-1 (qp) */
ISO8b2ISOqp: PROCEDURE

PARSE ARG string

IF string = '' THEN RETURN ''

string = ReplStr('','=DF',string) /* sz */
string = ReplStr('','=E4',string) /* ae */
string = ReplStr('','=F6',string) /* oe */
string = ReplStr('','=FC',string) /* ue */
string = ReplStr('','=C4',string) /* AE */
string = ReplStr('','=D6',string) /* OE */
string = ReplStr('','=DC',string) /* UE */

RETURN string


/* Procedure CountStr - Couting appearences of substrings */
CountStr: PROCEDURE

PARSE ARG substring, string

IF substring = '' THEN RETURN 0

precount = 0

substringlength = Length(substring)
PARSE VAR string . '' -(substringlength) lastsubstring
IF lastsubstring == substring THEN precount = 1

DO count = 0 BY 1 UNTIL string == ''
  PARSE VAR string . (substring) string
END

count = count + precount

RETURN count


/* Procedure ReplStr - Replacing substrings */
ReplStr: PROCEDURE

PARSE ARG oldsubstring, newsubstring, string

IF oldsubstring = '' THEN RETURN ''

DO i = 1 TO CountStr(oldsubstring, string)
  PARSE VAR string firstpart (oldsubstring) lastpart
  string = firstpart || newsubstring || lastpart
END

RETURN string
